"""genshinhelper entry point.

Using `python <project_package>` or `python -m <project_package>` command.
"""

if not __package__:
    import os
    import sys

    sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from .utils import get_mihoyo_app_cookie, log, _


def main(cookie):
    app_cookie = get_mihoyo_app_cookie(cookie)
    return app_cookie


if __name__ == "__main__":
    log.info(_('Converting cookie to mihoyo app cookie.'))
    raw_cookie = input(_('Please enter your cookie, similar to `account_id=xxxxxx; login_ticket=xxxxxx`: '))
    main(raw_cookie)



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

from .utils import log


class GenshinHelperException(Exception):
    """Base genshinhelper exception."""

    def __init__(self, message):
        super().__init__(message)
        log.error(message)



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

import uuid

from .exceptions import GenshinHelperException
from .utils import request, log, get_ds, nested_lookup, extract_subset_of_dict, merge_dicts, cookie_to_dict, today, _


def get_headers(oversea: bool = False, with_ds: bool = False, *args, **kwargs):
    ua_cn = 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) miHoYoBBS/{}'
    ua_os = 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) miHoYoBBSOversea/1.5.0'
    ua_default = ua_os if oversea else ua_cn
    app_version, client_type, ds = get_ds(*args, **kwargs)
    headers = {'User-Agent': ua_default.format(app_version)}
    if with_ds:
        headers.update({
            'x-rpc-device_id':
                str(uuid.uuid3(uuid.NAMESPACE_URL, ua_default)).replace(
                    '-', '').upper(),
            'x-rpc-client_type': client_type,
            'x-rpc-app_version': app_version,
            'DS': ds
        })
    return headers


class Client(object):
    def __init__(self, cookie: str = None):
        self.cookie = cookie_to_dict(cookie)
        self.headers = get_headers()
        self._roles_info = None
        self._sign_info = []
        self._rewards_info = []
        self._user_data = []
        self.api = 'https://api-takumi.mihoyo.com'
        self.act_id = None
        self.game_biz = None
        self.required_keys = {'region', 'game_uid', 'nickname', 'level', 'region_name'}

        self.roles_info_url = f'{self.api}/binding/api/getUserGameRolesByCookie' + '?game_biz={}'
        self.sign_info_url = None
        self.rewards_info_url = None
        self.sign_url = None

    @property
    def roles_info(self):
        if not self._roles_info:
            log.info(_('Preparing to get user game roles information ...'))
            url = self.roles_info_url.format(self.game_biz)
            response = request('get', url, headers=self.headers, cookies=self.cookie).json()
            log.debug(response)
            if response.get('retcode') != 0:
                raise GenshinHelperException(response.get('message'))

            raw_roles_info = nested_lookup(response, 'list', fetch_first=True)
            self._roles_info = [
                extract_subset_of_dict(i, self.required_keys)
                for i in raw_roles_info
            ]
        return self._roles_info

    @property
    def sign_info(self):
        ...
        return self._sign_info

    @property
    def rewards_info(self):
        if not self._rewards_info:
            log.info(_('Preparing to get monthly rewards information ...'))
            url = self.rewards_info_url
            response = request('get', url).json()
            self._rewards_info = nested_lookup(response, 'awards', fetch_first=True)
        return self._rewards_info

    @property
    def current_reward(self):
        sign_info = self.sign_info
        return [
            self.get_current_reward(i['total_sign_day'], i['is_sign'])
            for i in sign_info
        ]

    def get_current_reward(self, total_sign_day: int, is_sign: bool = False):
        rewards_info = self.rewards_info
        if isinstance(rewards_info[0], list):
            rewards_info = rewards_info[0]
        if is_sign:
            total_sign_day -= 1

        raw_current_reward = rewards_info[total_sign_day]
        return {'reward_' + k: v for k, v in raw_current_reward.items()}

    @property
    def user_data(self):
        sign_info = self.sign_info
        roles_info = self.roles_info
        current_reward = self.current_reward

        for i in range(len(sign_info)):
            d1 = roles_info[i]
            d2 = sign_info[i]
            d3 = current_reward[i]
            merged = merge_dicts(d1, d2, d3)
            self._user_data.append(merged)
        return self._user_data

    def sign(self):
        user_data = self.user_data
        log.info(_('Preparing to claim daily reward ...'))
        result = []
        for i in range(len(user_data)):
            user_data[i]['today'] = str(today())
            user_data[i]['status'] = _('👀 You have already checked-in')
            user_data[i]['addons'] = 'Olah! Odomu'
            user_data[i]['sign_response'] = None
            user_data[i]['end'] = ''
            total_sign_day = user_data[i]['total_sign_day']
            is_sign = user_data[i]['is_sign']

            if not is_sign:
                payload = {
                    'act_id': self.act_id,
                    'region': user_data[i]['region'],
                    'uid': user_data[i]['game_uid']
                }
                response = request(
                    'post',
                    self.sign_url,
                    headers=get_headers(with_ds=True),
                    json=payload, cookies=self.cookie).json()

                log.debug(response)
                user_data[i]['status'] = response.get('message', -1)
                user_data[i]['sign_response'] = response
                retcode = response.get('retcode', -1)
                # 0:      success
                # -5003:  already checked in
                if retcode == 0:
                    user_data[i]['total_sign_day'] = total_sign_day + 1
                    user_data[i]['is_sign'] = True
            result.append(user_data[i])

        self._user_data = result
        return result



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

import random
import time

from .core import Client, get_headers
from .utils import request, log, nested_lookup, extract_subset_of_dict, merge_dicts, cookie_to_dict, _


class YuanShen(Client):
    def __init__(self, cookie: str = None):
        super().__init__(cookie)
        self.act_id = 'e202009291139501'
        self.game_biz = 'hk4e_cn'
        self.required_keys.update({
            'total_sign_day', 'today', 'is_sign', 'first_bind',
            'current_primogems', 'current_mora'
        })

        self.sign_info_url = f'{self.api}/event/bbs_sign_reward/info?act_id={self.act_id}' + '&uid={}&region={}'
        self.rewards_info_url = f'{self.api}/event/bbs_sign_reward/home?act_id={self.act_id}'
        self.sign_url = f'{self.api}/event/bbs_sign_reward/sign'

        self._travelers_dairy = None
        self._daily_note = None
        self.travelers_dairy_url = 'https://hk4e-api.mihoyo.com/event/ys_ledger/monthInfo?bind_uid={}&bind_region={}&month={}&bbs_presentation_style=fullscreen&bbs_auth_required=true&mys_source=GameRecord'
        self.daily_note_url = f'{self.api}/game_record/app/genshin/api/dailyNote'

    @property
    def sign_info(self):
        if not self._sign_info:
            roles_info = self.roles_info
            self._sign_info = [
                self.get_sign_info(i['game_uid'], i['region'])
                for i in roles_info
            ]
        return self._sign_info

    def get_sign_info(self, uid: str, region: str):
        log.info(_('Preparing to get check-in information ...'))
        url = self.sign_info_url.format(uid, region)
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        data = nested_lookup(response, 'data', fetch_first=True)
        return extract_subset_of_dict(data, self.required_keys)

    @property
    def travelers_dairy(self):
        roles_info = self.roles_info
        self._travelers_dairy = [
            self.get_travelers_dairy(i['game_uid'], i['region'])
            for i in roles_info
        ]
        return self._travelers_dairy

    def get_travelers_dairy(self, uid: str, region: str, month: int = 0):
        log.info(_("Preparing to get traveler's dairy ..."))
        url = self.travelers_dairy_url.format(uid, region, month)
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        return nested_lookup(response, 'data', fetch_first=True)

    @property
    def month_dairy(self):
        raw_month_data = nested_lookup(self.travelers_dairy, 'month_data')
        return [
            extract_subset_of_dict(i, self.required_keys)
            for i in raw_month_data
        ]

    @property
    def daily_note(self):
        roles_info = self.roles_info
        self._daily_note = [
            self.get_daily_note(i['game_uid'], i['region'])
            for i in roles_info
        ]
        return self._daily_note

    def get_daily_note(self, uid: str, region: str):
        log.info(_('Preparing to get Yuan Shen daily note ...'))
        url = self.daily_note_url
        payload = {
            'role_id': uid,
            'server': region
        }
        response = request('get', url, headers=get_headers(with_ds=True, new_ds=True, params=payload), params=payload, cookies=self.cookie).json()
        data = nested_lookup(response, 'data', fetch_first=True)
        return data if data else response


class Honkai3rd(Client):
    def __init__(self, cookie: str = None):
        super().__init__(cookie)
        self.act_id = 'ea20211026151532'
        self.game_biz = 'bh3_cn'
        self.required_keys.update({
            'total_sign_day', 'today', 'is_sign', 'first_bind',
            'month_hcoin', 'month_star'
        })

        self.rewards_info_url = f'{self.api}/common/eutheniav2/index?act_id={self.act_id}' + '&uid={}&region={}'
        self.sign_url = f'{self.api}/common/eutheniav2/sign'

        self._bh3_finance = None
        self.bh3_finance_url = 'https://api.mihoyo.com/bh3-weekly_finance/api/index?bind_uid={}&bind_region={}&game_biz=bh3_cn'

    @property
    def sign_info(self):
        if not self._sign_info:
            rewards_info = self.rewards_info
            for i in rewards_info:
                # 0: can not check in
                # 1: can check in
                # 2: already checked in
                current_day = len(i) - nested_lookup(i, 'status').count(0)
                is_sign = True if i[current_day - 1]['status'] == 2 else False
                self._sign_info.append({
                    'total_sign_day': nested_lookup(i, 'status').count(2),
                    'is_sign': is_sign
                })
        return self._sign_info

    @property
    def rewards_info(self):
        if not self._rewards_info:
            log.info(_('Preparing to get monthly rewards information ...'))
            roles_info = self.roles_info
            for i in roles_info:
                url = self.rewards_info_url.format(i['game_uid'], i['region'])
                response = request('get', url, headers=self.headers, cookies=self.cookie).json()
                log.debug(response)
                self._rewards_info.append(nested_lookup(response, 'list', fetch_first=True))
        return self._rewards_info

    @property
    def bh3_finance(self):
        roles_info = self.roles_info
        self._bh3_finance = [
            self.get_bh3_finance(i['game_uid'], i['region'])
            for i in roles_info
        ]
        return self._bh3_finance

    # Requires the game roles level greater than 25
    def get_bh3_finance(self, uid: str, region: str):
        log.info(_('Preparing to get Honkai 3rd finance ...'))
        url = self.bh3_finance_url.format(uid, region)
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        return nested_lookup(response, 'data', fetch_first=True)

    @property
    def month_finance(self):
        bh3_finance = self.bh3_finance
        return [
            extract_subset_of_dict(i, self.required_keys)
            for i in bh3_finance
        ]


class MysDailyMissions(object):
    def __init__(self, cookie: str = None):
        self.cookie = cookie_to_dict(cookie)
        self.api = 'https://bbs-api.mihoyo.com'
        self.state_url = f'{self.api}/apihub/sapi/getUserMissionsState'
        self.sign_url = f'{self.api}/apihub/sapi/signIn' + '?gids={}'
        self.post_list_url = f'{self.api}/post/api/getForumPostList?&is_good=false&is_hot=false&page_size=20&sort_type=1' + '&forum_id={}'
        self.post_full_url = f'{self.api}/post/api/getPostFull' + '?post_id={}'
        self.upvote_url = f'{self.api}/apihub/sapi/upvotePost'
        self.share_url = f'{self.api}/apihub/api/getShareConf?entity_type=1' + '&entity_id={}'

        self._missions_state = None
        self._posts = None

        self.game_ids_dict = {1: '崩坏3', 2: '原神', 3: '崩坏2', 4: '未定事件簿', 5: '大别野', 6: '崩坏: 星穹铁道'}
        self.forum_ids_dict = {1: '崩坏3', 26: '原神', 30: '崩坏2', 37: '未定事件簿', 34: '大别野', 52: '崩坏: 星穹铁道'}
        self.game_ids = list(self.game_ids_dict.keys())
        self.forum_ids = list(self.forum_ids_dict.keys())
        self.result = {
            'sign': [],
            'view': [],
            'upvote': [],
            'share': []
        }

    @property
    def headers(self):
        headers = get_headers(with_ds=True, ds_type='android')
        headers.update({
            'User-Agent': 'okhttp/4.8.0',
            'Referer': 'https://app.mihoyo.com',
            'x-rpc-channel': 'miyousheluodi'
        })
        return headers

    @property
    def missions_state(self):
        log.info(_('Preparing to get user missions state ...'))
        url = self.state_url
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        data = nested_lookup(response, 'data')
        states = nested_lookup(response, 'states', fetch_first=True)
        _missions_state = {
            i['mission_key']: i['is_get_award'] for i in states if i['mission_id'] in (58, 59, 60, 61)
        }
        self._missions_state = {
            'total_points': nested_lookup(data, 'total_points', fetch_first=True),
            'is_sign': _missions_state.get('continuous_sign', False),
            'is_view': _missions_state.get('view_post_0', False),
            'is_upvote': _missions_state.get('post_up_0', False),
            'is_share': _missions_state.get('share_post_0', False)
        }
        return self._missions_state

    def sign(self, game_id: int = None):
        if not game_id:
            game_id = random.choice(self.game_ids)
        if game_id not in self.game_ids:
            raise ValueError(f'The value of game_id is one of {self.game_ids}')

        log.info(_('Preparing to check-in for {} ...').format(self.game_ids_dict[game_id]))
        url = self.sign_url.format(game_id)
        response = request('post', url, headers=self.headers, cookies=self.cookie).json()
        message = response.get('message')
        result = {'name': self.game_ids_dict[game_id], 'message': message}
        self.result['sign'].append(result)
        return result

    def get_posts(self, forum_id: int = None):
        if not forum_id:
            forum_id = random.choice(self.forum_ids)
        if forum_id not in self.forum_ids:
            raise ValueError(f'The value of forum_id is one of {self.forum_ids}')

        log.info(_('Preparing to get posts of {} ...').format(self.forum_ids_dict[forum_id]))
        url = self.post_list_url.format(forum_id)
        response = request('get', url).json()
        post_list = nested_lookup(response, 'list', fetch_first=True)
        posts = [{
            'post_id': nested_lookup(post, 'post_id', fetch_first=True),
            'title': nested_lookup(post, 'subject', fetch_first=True)
        } for post in post_list]
        log.info(_('Successfully get {} posts').format(len(posts)))
        return posts

    def view_post(self, post: dict):
        log.info(_('Preparing to view post {} ...').format(post['title']))
        time.sleep(3)
        url = self.post_full_url.format(post['post_id'])
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        message = response.get('message')
        log.info(message)
        result = {'title': post['title'], 'message': message}
        self.result['view'].append(result)
        return result

    def upvote_post(self, post: dict):
        log.info(_('Preparing to upvote post {} ...').format(post['title']))
        time.sleep(3)
        url = self.upvote_url
        data = {'post_id': post['post_id'], 'is_cancel': False}
        response = request('post', url, json=data, headers=self.headers, cookies=self.cookie).json()
        message = response.get('message')
        log.info(message)
        result = {'title': post['title'], 'message': message}
        self.result['upvote'].append(result)
        return result

    def share_post(self, post: dict):
        log.info(_('Preparing to share post {} ...').format(post['title']))
        url = self.share_url.format(post['post_id'])
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        message = response.get('message')
        log.info(message)
        result = {'title': post['title'], 'message': message}
        self.result['share'].append(result)
        return result

    def run(self, forum_id: int = None):
        state = self.missions_state
        [self.sign(i) for i in self.game_ids if not state['is_sign']]

        posts = self.get_posts(forum_id)
        [self.view_post(i) for i in random.sample(posts[0:5], 3) if not state['is_view']]
        [self.upvote_post(i) for i in random.sample(posts[5:17], 10) if not state['is_upvote']]
        [self.share_post(i) for i in random.sample(posts[-3:-1], 1) if not state['is_share']]

        state = self.missions_state
        self.result = merge_dicts(state, self.result)
        return self.result



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

from .utils import request

headers = {
    'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 MicroMessenger/8.0.1(0x1800012a) NetType/4G Language/zh_CN'
}


def check_jfsc(token):
    url = 'http://ysjfsc.mihoyo.com/api/SignIn/checkSign'
    payload = {'token': token}
    response = request('get', url, headers=headers, params=payload).json()
    return True if response.get('is_sign') else False


def sign_jfsc(token):
    url = 'http://ysjfsc.mihoyo.com/api/SignIn/sign'
    payload = {'token': token}
    return request('post', url, headers=headers, data=payload).json()



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

import re

from bs4 import BeautifulSoup

from .utils import request, nested_lookup, cookie_to_dict


class Weibo(object):
    def __init__(self, params: str = None, cookie: str = None):
        """
        params: s=xxxxxx; gsid=xxxxxx; aid=xxxxxx; from=xxxxxx
        """
        self.params = cookie_to_dict(params.replace('&', ';')) if params else None
        self.cookie = cookie_to_dict(cookie)

        self.container_id = '100808fc439dedbb06ca5fd858848e521b8716'
        self.ua = 'WeiboOverseas/4.4.6 (iPhone; iOS 14.0.1; Scale/2.00)'
        self.headers = {'User-Agent': self.ua}
        self.follow_data_url = 'https://api.weibo.cn/2/cardlist'
        self.sign_url = 'https://api.weibo.cn/2/page/button'
        self.event_url = f'https://m.weibo.cn/api/container/getIndex?containerid={self.container_id}_-_activity_list'
        self.mybox_url = 'https://ka.sina.com.cn/html5/mybox'
        self.draw_url = 'https://ka.sina.com.cn/innerapi/draw'
        self._follow_data = []

    @property
    def follow_data(self):
        if not self._follow_data:
            url = self.follow_data_url
            self.params['containerid'] = '100803_-_followsuper'
            # turn off certificate verification
            response = request('get', url, params=self.params, headers=self.headers, cookies=self.cookie, verify=False).json()
            card_group = nested_lookup(response, 'card_group', fetch_first=True)
            follow_list = [i for i in card_group if i['card_type'] == '8']
            for i in follow_list:
                action = nested_lookup(i, 'action', fetch_first=True)
                request_url = ''.join(
                    re.findall('request_url=(.*)%26container', action)) if action else None
                follow = {
                    'name': nested_lookup(i, 'title_sub', fetch_first=True),
                    'level': int(re.findall('\d+', i['desc1'])[0]),
                    'is_sign': False if nested_lookup(i, 'name', fetch_first=True) == '签到' else True,
                    'request_url': request_url
                }
                self._follow_data.append(follow)

            self._follow_data.sort(key=lambda k: (k['level']), reverse=True)
        return self._follow_data

    def sign(self):
        result = []
        for follow in self.follow_data:
            if not follow['is_sign']:
                url = self.sign_url
                self.params['request_url'] = follow['request_url']
                if self.params.get('containerid'):
                    del self.params['containerid']
                # turn off certificate verification
                response = request('get', url, params=self.params, headers=self.headers, cookies=self.cookie, verify=False).json()
                follow['sign_response'] = response
                if int(response.get('result', -1)) == 1:
                    follow['is_sign'] = True
                    follow['request_url'] = None

            result.append(follow)
        return result

    @property
    def event_list(self):
        url = self.event_url
        response = request('get', url).json()
        return nested_lookup(response, 'group', fetch_first=True)

    def check_event(self):
        return True if self.event_list else False

    def get_event_gift_ids(self):
        return [
            i
            for event in self.event_list
            for i in re.findall(r'gift/(\d*)', str(event['scheme']))
        ]

    def get_mybox_codes(self):
        url = self.mybox_url
        response = request('get', url, headers=self.headers, cookies=self.cookie, allow_redirects=False)
        if response.status_code != 200:
            raise Exception(
                'Failed to get my box codes: '
                'The cookie seems to be invalid, please re-login to https://ka.sina.com.cn'
            )

        response.encoding = 'utf-8'
        soup = BeautifulSoup(response.text, 'html.parser')
        # print(soup.prettify())
        boxs = soup.find_all(class_='giftbag')
        mybox_codes = []
        for box in boxs:
            item = {
                'id': box.find(class_='deleBtn').get('data-itemid'),
                'title': box.find(class_='title itemTitle').text,
                'code': box.find('span').parent.contents[1]
            }
            mybox_codes.append(item)
        return mybox_codes

    def unclaimed_gift_ids(self):
        event_gift_ids = self.get_event_gift_ids()
        mybox_gift_ids = [item.get('id') for item in self.get_mybox_codes()]
        return [i for i in event_gift_ids if i not in mybox_gift_ids]

    def get_code(self, id: str):
        url = self.draw_url
        self.headers.update({
            'Referer': f'https://ka.sina.com.cn/html5/gift/{id}'
        })
        data = {'gid': 10725, 'itemId': id, 'channel': 'wblink'}
        response = request('get', url, params=data, headers=self.headers, cookies=self.cookie).json()
        code = nested_lookup(response, 'kahao')

        result = {'success': True, 'id': id, 'code': code} if code else {'success': False, 'id': id, 'response': response}
        return result



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

from .core import Client, get_headers
from .utils import request, log, nested_lookup, extract_subset_of_dict, config, _

_LANG_DICT = {
    'zh': 'zh-cn',
    'en': 'en-us'
}


class Genshin(Client):
    def __init__(self, cookie: str = None):
        super().__init__(cookie)
        self.headers = get_headers(oversea=True)
        self.api = 'https://hk4e-api-os.mihoyo.com'
        self.act_id = 'e202102251931481'
        self.game_biz = 'hk4e_global'
        self.required_keys.update({
            'total_sign_day', 'today', 'is_sign', 'first_bind',
            'current_primogems', 'current_mora'
        })

        self.lang = _LANG_DICT.get(config.LANGUAGE, '')
        self.roles_info_url = 'https://api-os-takumi.mihoyo.com/binding/api/getUserGameRolesByCookie?game_biz={}'
        self.sign_info_url = f'{self.api}/event/sol/info?lang={self.lang}&act_id={self.act_id}'
        self.rewards_info_url = f'{self.api}/event/sol/home?lang={self.lang}&act_id={self.act_id}'
        self.sign_url = f'{self.api}/event/sol/sign?lang={self.lang}'

        self._travelers_dairy = None
        self.travelers_dairy_url = f'{self.api}/event/ysledgeros/month_info?lang={self.lang}&' + 'uid={}&region={}&month={}'

    @property
    def sign_info(self):
        if not self._sign_info:
            log.info(_('Preparing to get check-in information ...'))
            url = self.sign_info_url
            response = request('get', url, headers=self.headers, cookies=self.cookie).json()
            log.debug(response)
            data = nested_lookup(response, 'data', fetch_first=True)
            if data:
                del data['region']
            self._sign_info.append(extract_subset_of_dict(data, self.required_keys))
        return self._sign_info

    @property
    def travelers_dairy(self):
        roles_info = self.roles_info
        self._travelers_dairy = [
            self.get_travelers_dairy(i['game_uid'], i['region'])
            for i in roles_info
        ]
        return self._travelers_dairy

    def get_travelers_dairy(self, uid: str, region: str, month: int = 0):
        log.info(_("Preparing to get traveler's dairy ..."))
        url = self.travelers_dairy_url.format(uid, region, month)
        response = request('get', url, headers=self.headers, cookies=self.cookie).json()
        log.debug(response)
        return nested_lookup(response, 'data', fetch_first=True)

    @property
    def month_dairy(self):
        raw_month_data = nested_lookup(self.travelers_dairy, 'month_data')
        return [
            extract_subset_of_dict(i, self.required_keys)
            for i in raw_month_data
        ]



"""Utilities.
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

import datetime
import gettext
import hashlib
import json
import logging
import os
import random
import string
import time
from urllib.parse import urlencode

import requests

from genshinhelper import config

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(levelname)s %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S')

log = logger = logging

_localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
_translate = gettext.translation(
    'genshinhelper', _localedir, languages=[config.LANGUAGE], fallback=True)
_ = _translate.gettext

MESSAGE_TEMPLATE = _('''
    {today:#^18}
    🔅{nickname} {level} {region_name}
    Today's reward: {reward_name} × {reward_cnt}
    Total monthly check-ins: {total_sign_day} days
    Status: {status}
    {addons}
    {end:#^18}''')

DAIRY_TEMPLATE = _('''Traveler month {month} diary
    💠primogems: {current_primogems}
    🌕mora: {current_mora}''')

FINANCE_TEMPLATE = _('''Captain month {month} finance
    💎hcoin: {month_hcoin}
    🔮star: {month_star}''')


def set_lang(lang=None):
    if lang:
        os.environ['LANGUAGE'] = lang


def today():
    return datetime.date.today()


def month():
    return today().month


def get_mihoyo_app_cookie(cookie):
    if 'stoken' in cookie:
        return cookie

    cookie_dict = cookie_to_dict(cookie)
    stuid = cookie_dict['account_id']
    login_ticket = cookie_dict['login_ticket']
    url = 'https://api-takumi.mihoyo.com/auth/api/getMultiTokenByLoginTicket?uid={}&login_ticket={}&token_types=3'.format(stuid, login_ticket)
    response = request('get', url).json()
    list = nested_lookup(response, 'list', fetch_first=True)
    stoken = nested_lookup([i for i in list if i['name'] == 'stoken'], 'token', fetch_first=True)
    if not stoken:
        log.error(_('Failed to convert:\n{response}').format(response=response))
        return

    app_cookie = f'stuid={stuid}; stoken={stoken}; login_ticket={login_ticket}'
    log.info(_(f'Successful conversion!\n{app_cookie}').format(app_cookie=app_cookie))
    return app_cookie


def minutes_to_hours(minutes):
    minutes = int(minutes)
    if minutes < 0:
        raise ValueError('Input number cannot be negative')

    return {'hour': int(minutes / 60), 'minute': minutes % 60}


def get_cookies(cookies: str = None):
    if '#' in cookies:
        return cookies.split('#')
    elif isinstance(cookies, list):
        return cookies
    elif '{' in cookies:
        return json.loads(cookies)
    else:
        return cookies.splitlines()


def extract_cookie(name: str, cookie: str):
    if name not in cookie:
        raise Exception(
            _('Failed to extract cookie: The cookie does not contain the `{name}` field.').format(
                name=name
            )
        )
    return cookie.split(f'{name}=')[1].split(';')[0]


def cookie_to_dict(cookie):
    if cookie and '=' in cookie:
        cookie = dict([line.strip().split('=', 1) for line in cookie.split(';')])
    return cookie


def merge_dicts(*dict_args):
    result = {}
    for d in dict_args:
        result.update(d)
    return result


def extract_subset_of_dict(raw_dict, keys):
    subset = {}
    if isinstance(raw_dict, dict):
        subset = {key: value for key, value in raw_dict.items() if key in keys}
    return subset


def nested_lookup(obj, key, with_keys=False, fetch_first=False):
    result = list(_nested_lookup(obj, key, with_keys=with_keys))
    if with_keys:
        values = [v for k, v in _nested_lookup(obj, key, with_keys=with_keys)]
        result = {key: values}
    if fetch_first:
        result = result[0] if result else result
    return result


def _nested_lookup(obj, key, with_keys=False):
    if isinstance(obj, list):
        for i in obj:
            yield from _nested_lookup(i, key, with_keys=with_keys)

    if isinstance(obj, dict):
        for k, v in obj.items():
            if key == k:
                if with_keys:
                    yield k, v
                else:
                    yield v

            if isinstance(v, list) or isinstance(v, dict):
                yield from _nested_lookup(v, key, with_keys=with_keys)


def get_ds(ds_type: str = None, new_ds: bool = False, data: dict = None, params: dict = None):
    # 1:  ios
    # 2:  android
    # 4:  pc web
    # 5:  mobile web
    def new():
        t = str(int(time.time()))
        r = str(random.randint(100000, 200000))
        b = json.dumps(data) if data else ''
        q = urlencode(params) if params else ''
        c = _hexdigest(f'salt={salt}&t={t}&r={r}&b={b}&q={q}')
        return f'{t},{r},{c}'

    def old():
        t = str(int(time.time()))
        r = ''.join(random.sample(string.ascii_lowercase + string.digits, 6))
        c = _hexdigest(f'salt={salt}&t={t}&r={r}')
        return f'{t},{r},{c}'

    app_version = '2.3.0'
    client_type = '5'
    salt = 'h8w582wxwgqvahcdkpvdhbh2w9casgfl'
    ds = old()
    if ds_type == '2' or ds_type == 'android':
        app_version = '2.8.0'
        client_type = '2'
        salt = 'dmq2p7ka6nsu0d3ev6nex4k1ndzrnfiy'
        ds = old()
    if new_ds:
        app_version = '2.11.1'
        client_type = '5'
        salt = 'xV8v4Qu54lUKrEYFZkJhB8cuOh9Asafs'
        ds = new()

    return app_version, client_type, ds


def _hexdigest(text):
    md5 = hashlib.md5()
    md5.update(text.encode())
    return md5.hexdigest()


def request(*args, **kwargs):
    is_retry = True
    count = 0
    max_retries = 3
    sleep_seconds = 5
    while is_retry and count <= max_retries:
        try:
            s = requests.Session()
            response = s.request(*args, **kwargs)
            is_retry = False
        except Exception as e:
            if count == max_retries:
                raise e
            log.error(_('Request failed: {}').format(e))
            count += 1
            log.info(
                _('Trying to reconnect in {} seconds ({}/{})...').format(
                    sleep_seconds, count, max_retries))
            time.sleep(sleep_seconds)
        else:
            return response




import subprocess
from pathlib import Path

from setuptools import find_packages, setup

long_description = Path('README.md').read_text(encoding='utf-8')


def load_requirements(filename):
    with Path(filename).open() as f:
        return [line.strip() for line in f if not line.startswith('#')]


def create_mo_files():
    data_files = []
    locale_dir = Path('genshinhelper/locale')
    po_dirs = [lang / 'LC_MESSAGES' for lang in locale_dir.iterdir() if lang.is_dir()]
    for dir in po_dirs:
        mo_files = []
        po_files = [file for file in dir.iterdir() if file.suffix == '.po']
        for po_file in po_files:
            mo_file = po_file.with_suffix('.mo')
            msgfmt_cmd = ['msgfmt', po_file, '-o', mo_file]
            subprocess.run(msgfmt_cmd)
            mo_files.append(str(mo_file))
        data_files.append((str(dir), mo_files))
    return data_files


__version__ = None
with open('genshinhelper/__version__.py', encoding='utf-8') as f:
    exec(f.read())

if not __version__:
    print('Could not find __version__ from genshinhelper/__version__.py')
    exit(-1)

setup(
    name='genshinhelper',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/y1ndan/genshinhelper2',
    license='GPLv3',
    author='y1ndan',
    author_email='i@yindan.me',
    description='A Python library for miHoYo bbs and HoYoLAB Community.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='原神 签到 mihoyo hoyolab genshin genshin-impact check-in weibo',
    include_package_data=True,
    data_files=create_mo_files(),
    install_requires=load_requirements('requirements.txt'),
    tests_require=['pytest'],
    classifiers=[
        # As from https://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        # 'Development Status :: 4 - Beta',
        'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # 'Programming Language :: Python :: 2',
        # 'Programming Language :: Python :: 2.3',
        # 'Programming Language :: Python :: 2.4',
        # 'Programming Language :: Python :: 2.5',
        # 'Programming Language :: Python :: 2.6',
        # 'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.0',
        # 'Programming Language :: Python :: 3.1',
        # 'Programming Language :: Python :: 3.2',
        # 'Programming Language :: Python :: 3.3',
        # 'Programming Language :: Python :: 3.4',
        # 'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Systems Administration',
    ],
    entry_points={
        'console_scripts': [
            'genshinhelper=genshinhelper.__main__:main'
        ]
    },
    python_requires='>=3.6',
)



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

from .__version__ import __version__
from .cloudgenshin import get_cloudgenshin_free_time
from .hoyolab import Genshin
from .mihoyo import (
    YuanShen,
    Honkai3rd,
    MysDailyMissions,
)
from .jfsc import check_jfsc, sign_jfsc
from .utils import (
    request,
    today,
    month,
    get_mihoyo_app_cookie,
    set_lang,
)
from .weibo import Weibo



"""
@Project   : genshinhelper
@Author    : y1ndan
@Blog      : https://www.yindan.me
@GitHub    : https://github.com/y1ndan
"""

from .utils import request


def get_cloudgenshin_free_time(headers):
    url = 'https://api-cloudgame.mihoyo.com/hk4e_cg_cn/wallet/wallet/get'
    return request('get', url, headers=headers).json()
